Option Strict On

Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Public Class clsEngine
  Private p_D3DDevice As Device
  Private p_target As Control

  Private Init As Boolean = False

  Public Sub New(ByVal target As Control, ByVal width As Integer, ByVal height As Integer, ByVal depth As Integer)
    Dim pp As New PresentParameters()
    With pp
      .BackBufferCount = 1
      .BackBufferWidth = width
      .BackBufferHeight = height
      Select Case depth
      Case 16
        .BackBufferFormat = Format.R5G6B5
      Case 32
        .BackBufferFormat = Format.X8R8G8B8
      End Select

      .SwapEffect = SwapEffect.Copy
      .PresentationInterval = PresentInterval.Immediate

      p_target = target
    End With
    init = InitDevice(pp)
  End Sub

  Public Sub New(ByVal target As Control)
    Dim pp As New PresentParameters()
    With pp
      .Windowed = True

      .SwapEffect = SwapEffect.Discard
      .BackBufferCount = 1
      .BackBufferFormat = Manager.Adapters(0).CurrentDisplayMode().Format
      .BackBufferWidth = target.Width
      .BackBufferHeight = target.Height

      p_target = target
    End With
    init = InitDevice(pp)
  End Sub

  Private Function InitDevice(ByVal pp As PresentParameters) As Boolean
    Dim D3DCaps As Caps = Manager.GetDeviceCaps(0, DeviceType.Hardware)
    Dim CreateFlgs As CreateFlags

    pp.AutoDepthStencilFormat = DepthFormat.D16
    pp.EnableAutoDepthStencil = True

    If D3DCaps.DeviceCaps.SupportsHardwareTransformAndLight() And _
        D3DCaps.DeviceCaps.SupportsPureDevice Then
      CreateFlgs = CreateFlags.HardwareVertexProcessing Or _
        CreateFlags.PureDevice
    ElseIf D3DCaps.DeviceCaps.SupportsHardwareTransformAndLight() Then
      CreateFlgs = CreateFlags.HardwareVertexProcessing
    Else
      CreateFlgs = CreateFlags.SoftwareVertexProcessing
    End If
    CreateFlgs = CreateFlgs Or CreateFlags.MultiThreaded

    Try
      p_D3DDevice = New Device(0, DeviceType.Hardware, p_target, CreateFlgs, pp)
    Catch dxerr As DirectXException
      Return False
    Catch
      Return False
    End Try

    Return True
  End Function
End Class
